package
{
	import enums.CallState;
	
	import events.CallErrorEvent;
	import events.CallStateEvent;
	import events.DtmfEvent;
	import events.IncomingCallEvent;
	import events.PhoneLineEvent;
	import events.RegistrationEvent;
	
	import flash.events.EventDispatcher;
	import flash.media.Microphone;
	
	import ozeki.mediaGateway.*;
	
	/**A class what represent a SIPClient and hold the contact with the server.*/
	public class SIPClient extends EventDispatcher
	{
		private var mediaConnection:MediaConnection;
		private var mediaStreamSender:MediaStreamSender;
		private var mediaStreamReceiver:MediaStreamReceiver;
		
		private var _displayName:String;
		public function SIPClient()
		{
			mediaConnection = new MediaConnection("localhost/SIPGateway");
			mediaConnection.client = this;
			
			mediaConnection.addEventListener(ConnectionEvent.CONNECTION_STATE_CHANGED, connectionStateChanged);
		}
		/**Connects to the server.*/
		public function connect():void
		{
			mediaConnection.connect();
		}
		/**Calls a phonenumber.
		 * @param The dial number.*/
		public function call(dialNumber:String):void
		{
			mediaConnection.invokeOnConnection("Call", dialNumber);
		}
		/**If the call is InCall state ends this call.*/
		public function hangUp():void
		{
			mediaConnection.invokeOnConnection("HangUp");
		}
		/**Reject the incoming call*/
		public function reject():void
		{
			mediaConnection.invokeOnConnection("Reject");
		}
		/**Accepts an incoming call request.*/
		public function accept():void
		{
			mediaConnection.invokeOnConnection("Accept");
		}
		/**Stats to send a DTMF sign.*/
		public function startDtmf(dtmf:int):void
		{
			mediaConnection.invokeOnConnection("StartDtmf", dtmf);
		}
		/** Stops to send a DTMF sign.*/
		public function stopDtmf(dtmf:int):void
		{
			mediaConnection.invokeOnConnection("StopDtmf", dtmf);
		}
		/**Indicates the call state has changed.
		 * @param The current state of the call.*/
		public function OnCallStateChanged(callState:int):void
		{
			if(callState == CallState.COMPLETED)
			{
				closeStreams();
			}
			
			dispatchEvent(new CallStateEvent(callState));
		}
		/**Indicates for the client, his call request ended with error.*/
		public function OnCallErrorOccurred(callError:int):void
		{
			dispatchEvent(new CallErrorEvent(callError));
		}
		/**Indicates for the client, he has an incoming call. Invoke the client side OnIncomingCall method.
		 * @param The request owner's phonenumber.*/
		public function OnIncomingCall(phoneNumber:String):void
		{
			dispatchEvent(new IncomingCallEvent(phoneNumber));
		}
		/**Indicates the SIPClient connection to the server is success or not.
		 * @param The current state of registration process.*/
		public function OnRegistrationStateChanged(registrationState:int):void
		{
			dispatchEvent(new RegistrationEvent(registrationState));
		}
		
		/**Indicates for the client, his phoneline state has changed.*/
		public function OnPhoneLineStateChanged(phoneLineState:int):void
		{
			dispatchEvent(new PhoneLineEvent(phoneLineState));
		}
		/**Indicates the SIPClient registered username.*/
		public function OnRegisteredUserNameReceived(displayName:String):void
		{
			this.displayName = displayName;
		}
		
		/**Indicates the incomning DTMF signs.*/
		public function OnDTMFReceived(dtmfSign:int):void
		{
			dispatchEvent(new DtmfEvent(dtmfSign));
		}
		/**Registered SIP username*/
		[Bindable]
		public function get displayName():String
		{
			return _displayName;
		}
		
		public function set displayName(value:String):void
		{
			_displayName = value;
		}
		
		/**Starts to send the audio data to the server and starts to play other party's media data.
		 */
		public function OnInCall(publishStreamName:String, playStreamName:String):void
		{
			mediaStreamSender = new MediaStreamSender(mediaConnection);
			mediaStreamSender.attachMicrophone(Microphone.getMicrophone());
			mediaStreamSender.publish(publishStreamName);
			
			mediaStreamReceiver = new MediaStreamReceiver(mediaConnection);
			mediaStreamReceiver.play(playStreamName);
		}
		
		private function connectionStateChanged(event:ConnectionEvent):void
		{
			dispatchEvent(event);
		}
		
		private function closeStreams():void
		{
			if(mediaStreamReceiver)
			{
				mediaStreamReceiver.close();
				mediaStreamReceiver = null;
			}
			if(mediaStreamSender)
			{
				mediaStreamSender.close();
				mediaStreamSender = null;
			}
		}
	}
}